@@ -14,13 +14,15 @@
 
 The test case encapsulates the request headers and body and expected
 response headers and body. When the test is run an HTTP request is
-made using urllib3. Assertions are made against the response.
+made using httpx. Assertions are made against the response.
 """
 
 from collections import OrderedDict
 import copy
 import functools
 from http import cookies
+import httpx
+import numbers
 import os
 import re
 import sys
@@ -29,8 +31,6 @@ import unittest
 from unittest import result as unitresult
 import urllib.parse as urlparse
 
-import wsgi_intercept
-
 from gabbi import __version__
 from gabbi import exception
 from gabbi.handlers import base
@@ -79,6 +79,7 @@ BASE_TEST = {
     'use_prior_test': True,
     'disable_response_handler': False,
     'timeout': 30,
+    "http_version": 1,
 }
 
 
@@ -276,7 +277,7 @@ class HTTPTestCase(unittest.TestCase):
         Let KeyError raise if variable not present. Capture
         the 'cast' if any.
         """
-        environ_name = match.group('arg')
+        environ_name = match.group('arg1') or match.group('arg2')
         self.cast = match.group('cast')
         return os.environ[environ_name]
 
@@ -292,7 +293,7 @@ class HTTPTestCase(unittest.TestCase):
 
     def _cookie_replacer(self, match):
         """Replace a regex match with the cookie of a previous response."""
-        case = match.group('case')
+        case = match.group('case1') or match.group('case2')
         if case:
             referred_case = self.history[case]
         else:
@@ -315,8 +316,8 @@ class HTTPTestCase(unittest.TestCase):
 
     def _header_replacer(self, match):
         """Replace a regex match with the value of a prior header."""
-        header_key = match.group('arg')
-        case = match.group('case')
+        header_key = match.group('arg1') or match.group('arg2')
+        case = match.group('case1') or match.group('case2')
         if case:
             referred_case = self.history[case]
         else:
@@ -345,7 +346,7 @@ class HTTPTestCase(unittest.TestCase):
 
     def _url_replacer(self, match):
         """Replace a regex match with the value of a previous url."""
-        case = match.group('case')
+        case = match.group('case1') or match.group('case2')
         if case:
             referred_case = self.history[case]
         else:
@@ -364,7 +365,7 @@ class HTTPTestCase(unittest.TestCase):
 
     def _location_replacer(self, match):
         """Replace a regex match with the value of a previous location."""
-        case = match.group('case')
+        case = match.group('case1') or match.group('case2')
         if case:
             referred_case = self.history[case]
         else:
@@ -423,7 +424,13 @@ class HTTPTestCase(unittest.TestCase):
                                     parsed_url.path, query_string, ''))
 
     _history_regex = (
-        r"(?:\$HISTORY\[(?P<quote1>['\"])(?P<case>.+?)(?P=quote1)\]\.)??"
+        r"(?:\$HISTORY"
+        r"("
+        r"\['(?P<case1>[^']+?)'\]"
+        "|"
+        r'\["(?P<case2>[^"]+?)"\]'
+        r")"
+        r"\.)??"
     )
 
     @staticmethod
@@ -432,7 +439,11 @@ class HTTPTestCase(unittest.TestCase):
         case = HTTPTestCase._history_regex
         return (
             r"%s\$%s(:(?P<cast>\w+))?"
-            r"\[(?P<quote>['\"])(?P<arg>.+?)(?P=quote)\]"
+            r"("
+            r"\['(?P<arg1>[^']+?)'\]"
+            "|"
+            r'\["(?P<arg2>[^"]+?)"\]'
+            r")"
             % (case, key))
 
     @staticmethod
@@ -458,8 +469,8 @@ class HTTPTestCase(unittest.TestCase):
 
     def _response_replacer(self, match, preserve=False):
         """Replace a regex match with the value from a previous response."""
-        response_path = match.group('arg')
-        case = match.group('case')
+        response_path = match.group('arg1') or match.group('arg2')
+        case = match.group('case1') or match.group('case2')
         self.cast = match.group('cast')
         if not preserve and self.cast:
             raise RuntimeError(
@@ -492,29 +503,19 @@ class HTTPTestCase(unittest.TestCase):
         redirect=False,
         timeout=30,
     ):
-        """Run the http request and decode output.
-
-        The call to make the request will catch a WSGIAppError from
-        wsgi_intercept so that the real traceback from a catastrophic
-        error in the intercepted app can be examined.
-        """
+        """Run the http request and decode output."""
 
         if 'user-agent' not in (key.lower() for key in headers):
-            headers['user-agent'] = "gabbi/%s (Python urllib3)" % __version__
+            headers['user-agent'] = "gabbi/%s (Python httpx)" % __version__
 
-        try:
-            response, content = self.http.request(
-                url,
-                method=method,
-                headers=headers,
-                body=body,
-                redirect=redirect,
-                timeout=timeout,
-            )
-        except wsgi_intercept.WSGIAppError as exc:
-            # Extract and re-raise the wrapped exception.
-            raise (exc.exception_type, exc.exception_value,
-                   exc.traceback)
+        response, content = self.http.request(
+            url,
+            method=method,
+            headers=headers,
+            body=body,
+            redirect=redirect,
+            timeout=timeout,
+        )
 
         # Set headers and location attributes for follow on requests
         self.response = response
@@ -570,7 +571,7 @@ class HTTPTestCase(unittest.TestCase):
             test['name'], test['response_headers'])
 
         method = test['method'].upper()
-        headers = test['request_headers']
+        headers = self._format_headers_for_httpx(test['request_headers'])
 
         if test['data'] != '':
             body = self._test_data_to_string(
@@ -602,7 +603,11 @@ class HTTPTestCase(unittest.TestCase):
                     self._assert_response()
                     failure = None
                     break
-                except (AssertionError, utils.ConnectionRefused) as exc:
+                except (
+                    AssertionError,
+                    utils.ConnectionRefused,
+                    httpx.ReadTimeout
+                ) as exc:
                     failure = exc
 
                 count -= 1
@@ -742,3 +747,13 @@ class HTTPTestCase(unittest.TestCase):
             self.fail(msg)
         else:
             self.assertIn(expected, iterable)
+
+    def _format_headers_for_httpx(self, headers):
+        """`httpx` requires request headers to be strings"""
+        new_headers = {}
+        for key, val in headers.items():
+            if isinstance(val, numbers.Number):
+                new_headers[key] = str(val)
+            else:
+                new_headers[key] = val
+        return new_headers
