@@ -1,10 +1,11 @@
-#!/bin/bash -ex
+#!/usr/bin/env bash
+set -ex
 # Run the tests and confirm that the stuff we expect to skip or fail
 # does.
 
 # this would be somewhat less complex in bash4..
 shopt -s nocasematch
-[[ "${GABBI_SKIP_NETWORK:-false}" == "true" ]] && SKIP=12 || SKIP=2
+[[ "${GABBI_SKIP_NETWORK:-false}" == "true" ]] && SKIP=14 || SKIP=2
 [[ "${GABBI_SKIP_NETWORK:-false}" == "true" ]] && FAILS=15 || FAILS=16
 shopt -u nocasematch
 
