@@ -1,4 +1,5 @@
-#!/bin/bash
+#!/usr/bin/env bash
+set -ex
 # Run a test that confirms that a verbose test will output a response body
 # when there is no content-type.
 
